@@ -2,7 +2,7 @@
 
 __author__ = 'Staffan Malmgren'
 __email__ = 'staffan.malmgren@gmail.com'
-__version__ = "0.1.0"
+__version__ = "0.2.0"
 
 from .layeredconfig import LayeredConfig
 from .configsource import ConfigSource
@@ -14,3 +14,5 @@ from .commandline import Commandline
 from .environment import Environment
 from .plistfile import PListFile
 from .yamlfile import YAMLFile
+from .pyfile import PyFile
+from .etcdstore import EtcdStore
