@@ -8,28 +8,128 @@ test_layeredconfig
 Tests for `layeredconfig` module.
 """
 
+
 import os
 import logging
 import sys
-if sys.version_info < (2, 7, 0):  # pragma: no cover
-    import unittest2 as unittest
-else: 
-    import unittest
 import codecs
 from six import text_type as str
 from datetime import date, datetime
+import argparse
+import json
+from operator import itemgetter
 try:
     from collections import OrderedDict
 except ImportError:  # pragma: no cover
     # if on python 2.6
     from ordereddict import OrderedDict
 
+if sys.version_info < (2, 7, 0):  # pragma: no cover
+    import unittest2 as unittest
+else: 
+    import unittest
+import requests
 # The system under test
 from layeredconfig import (LayeredConfig, Defaults, INIFile, JSONFile,
-                           YAMLFile, PListFile, Environment, Commandline)
+                           YAMLFile, PListFile, PyFile, Environment,
+                           Commandline, EtcdStore)
 
 
-class TestConfigSourceHelper(object):
+class TestLayeredConfigHelper(object):
+
+    # Testcases for less-capable sources may override this
+    supported_types = (str, int, bool, list, date, datetime)
+    transforms = {}
+    supports_nesting = True
+
+    def _test_config_singlesection(self, cfg):
+        self.assertIs(type(cfg.home), str)
+        self.assertEqual(cfg.home, 'mydata')
+
+        int_type = int if int in self.supported_types else str
+        self.assertIs(type(cfg.processes), int_type)
+        self.assertEqual(cfg.processes, int_type(4))
+
+        bool_type = bool if bool in self.supported_types else str
+        # ugly hack to work around the case that etcd will transform a
+        # (string) value of "True" to "true"
+        bool_transform = self.transforms.get(bool, bool_type)
+        self.assertIs(type(cfg.force), bool_type)
+        self.assertEqual(cfg.force, bool_transform(True))
+
+        if list in self.supported_types:
+            list_type = list
+            list_want = ['foo', 'bar']
+        else:
+            list_type = str
+            list_want = "foo, bar"  # recommended list serialization
+        self.assertIs(type(cfg.extra), list_type)
+        self.assertEqual(cfg.extra, list_want)
+
+        if date in self.supported_types:
+            date_type = date
+            date_want = date(2014, 10, 15)
+        else:
+            date_type = str
+            date_want = "2014-10-15" # recommended date serialization
+        self.assertIs(type(cfg.expires), date_type)
+        self.assertEqual(cfg.expires, date_want)
+
+        if datetime in self.supported_types:
+            datetime_type = datetime
+            datetime_want = datetime(2014, 10, 15, 14, 32, 7)
+        else:
+            datetime_type = str
+            datetime_want = "2014-10-15 14:32:07"
+        self.assertIs(type(cfg.lastrun), datetime_type)
+        self.assertEqual(cfg.lastrun, datetime_want)
+
+    def _test_config_subsections(self, cfg):
+        self.assertEqual(cfg.home, 'mydata')
+        with self.assertRaises(AttributeError):
+            cfg.mymodule.home
+
+        int_type = int if int in self.supported_types else str
+        self.assertEqual(cfg.processes, int_type(4))
+        with self.assertRaises(AttributeError):
+            cfg.mymodule.processes
+
+        bool_type = bool if bool in self.supported_types else str
+        # ugly hack to work around the case that etcd will transform a
+        # (string) value of "True" to "true"
+        bool_transform = self.transforms.get(bool, bool_type)
+        self.assertEqual(cfg.force, bool_transform(True))
+        self.assertEqual(cfg.mymodule.force, bool_transform(False))
+
+        if list in self.supported_types:
+            list_type = list
+            list_want = ['foo', 'bar']
+            list_want_sub = ['foo', 'baz']
+        else:
+            list_type = str
+            list_want = "foo, bar"
+            list_want_sub = "foo, baz"
+        self.assertEqual(cfg.extra, list_want)
+        self.assertEqual(cfg.mymodule.extra, list_want_sub)
+
+        # not supported for INIFile
+        if self.supports_nesting:
+            self.assertEqual(cfg.mymodule.arbitrary.nesting.depth, 'works')
+
+        with self.assertRaises(AttributeError):
+            cfg.expires
+
+        if date in self.supported_types:
+            date_type = date
+            date_want = date(2014, 10, 15)
+        else:
+            date_type = str
+            date_want = "2014-10-15" # recommended date serialization
+        self.assertEqual(cfg.mymodule.expires, date_want)
+
+
+class TestConfigSourceHelper(TestLayeredConfigHelper):
+
     # First, a number of straightforward tests for any
     # ConfigSource-derived object. Concrete test classes should set up
     # self.simple and self.complex instances to match these.
@@ -54,7 +154,7 @@ class TestConfigSourceHelper(object):
         subsec = self.complex.subsection('mymodule')
         self.assertEqual(set(subsec.subsections()),
                          set(('arbitrary',)))
-        
+
     def test_has(self):
         for key in self.simple.keys():
             self.assertTrue(self.simple.has(key))
@@ -64,6 +164,9 @@ class TestConfigSourceHelper(object):
             self.assertTrue(self.simple.typed(key))
                 
     def test_get(self):
+        # FIXME: This test should be able to look at supported_types
+        # like test_singlesection and test_subsections do, so derived
+        # testcase classes don't need to override it.
         self.assertEqual(self.simple.get("home"), "mydata")
         self.assertEqual(self.simple.get("processes"), 4)
         self.assertEqual(self.simple.get("force"), True)
@@ -72,77 +175,18 @@ class TestConfigSourceHelper(object):
         self.assertEqual(self.simple.get("lastrun"),
                          datetime(2014, 10, 15, 14, 32, 7))
 
-
-class TestLayeredConfigHelper(object):
     # Then, two validation helpers for checking a complete
     # LayeredConfig object, where validation can be performed
     # different depending on the abilities of the source (eg. typing)
-    def _test_mainsection(self, cfg,
-                          int_type=int,
-                          bool_type=bool,
-                          list_type=list,
-                          date_type=date,
-                          datetime_type=datetime):
+    def test_config_singlesection(self):
+        # each subclass is responsible for creating a self.simple
+        # object of the type being tested
+        cfg = LayeredConfig(self.simple)
+        self._test_config_singlesection(cfg)
 
-        self.assertIs(type(cfg.home), str)
-        self.assertEqual(cfg.home, 'mydata')
-        self.assertIs(type(cfg.processes), int_type)
-        self.assertEqual(cfg.processes, int_type(4))
-        self.assertIs(type(cfg.force), bool_type)
-        self.assertEqual(cfg.force, bool_type(True))
-        self.assertIs(type(cfg.extra), list_type)
-        if list_type == str:
-            want = "foo, bar"  # recommended serialization of lists of strings
-        else:
-            want = ['foo', 'bar']
-        self.assertEqual(cfg.extra, want)
-        self.assertIs(type(cfg.expires), date_type)
-        if date_type == str:
-            self.assertEqual(cfg.expires, date_type(date(2014, 10, 15)))
-        else:
-            self.assertEqual(cfg.expires, date(2014, 10, 15))
-        self.assertIs(type(cfg.lastrun), datetime_type)
-        if datetime_type == str:
-            self.assertEqual(cfg.lastrun,
-                             datetime_type(datetime(2014, 10, 15, 14, 32, 7)))
-        else:
-            self.assertEqual(cfg.lastrun, datetime(2014, 10, 15, 14, 32, 7))
-
-    def _test_subsections(self, cfg,
-                          int_type=int,
-                          bool_type=bool,
-                          list_type=list,
-                          date_type=date,
-                          datetime_type=datetime,
-                          arbitrary_nesting=True):
-        self.assertEqual(cfg.home, 'mydata')
-        with self.assertRaises(AttributeError):
-            cfg.mymodule.home
-        self.assertEqual(cfg.processes, int_type(4))
-        with self.assertRaises(AttributeError):
-            cfg.mymodule.processes
-        self.assertEqual(cfg.force, bool_type(True))
-        self.assertEqual(cfg.mymodule.force, bool_type(False))
-        if list_type == str:
-            want = "foo, bar"  # recommended serialization of lists of strings
-        else:
-            want = ['foo', 'bar']
-        self.assertEqual(cfg.extra, want)
-        if list_type == str:
-            want = "foo, baz"
-        else:
-            want = ['foo', 'baz']
-        self.assertEqual(cfg.mymodule.extra, want)
-        # not supported for INIFile
-        if arbitrary_nesting:
-            self.assertEqual(cfg.mymodule.arbitrary.nesting.depth, 'works')
-        with self.assertRaises(AttributeError):
-            cfg.expires
-        if date_type == str:
-            self.assertEqual(cfg.mymodule.expires,
-                             date_type(date(2014, 10, 15)))
-        else:
-            self.assertEqual(cfg.mymodule.expires, date(2014, 10, 15))
+    def test_config_subsections(self):
+        cfg = LayeredConfig(self.complex)
+        self._test_config_subsections(cfg)
 
 
 # common helper
@@ -184,7 +228,7 @@ unique = True
         os.unlink("complex.ini")
 
 
-class TestDefaults(unittest.TestCase, TestConfigSourceHelper, TestLayeredConfigHelper):
+class TestDefaults(unittest.TestCase, TestConfigSourceHelper):
 
     simple = Defaults({'home': 'mydata',
                        'processes': 4,
@@ -208,19 +252,13 @@ class TestDefaults(unittest.TestCase, TestConfigSourceHelper, TestLayeredConfigH
                                  },
                         'extramodule': {'unique': True}})
 
-    def test_config(self):
-        cfg = LayeredConfig(self.simple)
-        self._test_mainsection(cfg)
-
-    def test_config_subsections(self):
-        cfg = LayeredConfig(self.complex)
-        self._test_subsections(cfg)
-
 
 class TestINIFile(TestINIFileHelper, unittest.TestCase,
-                  TestConfigSourceHelper,
-                  TestLayeredConfigHelper):
+                  TestConfigSourceHelper):
 
+    supported_types = (str,)
+    supports_nesting = False
+    
     def setUp(self):
         super(TestINIFile, self).setUp()
         self.simple = INIFile("simple.ini")
@@ -241,31 +279,12 @@ class TestINIFile(TestINIFileHelper, unittest.TestCase,
         for key in self.simple.keys():
             self.assertFalse(self.simple.typed(key))
 
-    # INIFile doesn't support nested subsections
+    # Override: INIFile doesn't support nested subsections
     def test_subsection_nested(self):
         subsec = self.complex.subsection('mymodule')
         self.assertEqual(set(subsec.subsections()),
                          set(()))
 
-    def test_inifile(self):
-        cfg = LayeredConfig(self.simple)
-        self._test_mainsection(cfg,
-                               int_type=str,
-                               bool_type=str,
-                               list_type=str,
-                               date_type=str,
-                               datetime_type=str)
-
-    def test_inifile_subsections(self):
-        cfg = LayeredConfig(self.complex)
-        self._test_subsections(cfg,
-                               int_type=str,
-                               bool_type=str,
-                               list_type=str,
-                               date_type=str,
-                               datetime_type=str,
-                               arbitrary_nesting=False)
-
     def test_inifile_default_as_root(self):
         # using a rootsection named DEFAULT triggers different
         # cascading-like behaviour in configparser.
@@ -296,7 +315,6 @@ class TestINIFile(TestINIFileHelper, unittest.TestCase,
 
         os.unlink("complex-otherroot.ini")
 
-
     def test_inifile_nonexistent(self):
         logging.getLogger().setLevel(logging.CRITICAL)
         cfg = LayeredConfig(INIFile("nonexistent.ini"))
@@ -349,9 +367,11 @@ unique = True
             got = fp.read().replace("\r\n", "\n")
         self.assertEqual(want, got)
 
-class TestJSONFile(unittest.TestCase, TestConfigSourceHelper,
-                   TestLayeredConfigHelper):
 
+class TestJSONFile(unittest.TestCase, TestConfigSourceHelper):
+
+    supported_types = (str, int, bool, list)
+    
     def setUp(self):
         with open("simple.json", "w") as fp:
             fp.write("""
@@ -404,25 +424,6 @@ class TestJSONFile(unittest.TestCase, TestConfigSourceHelper,
             else:
                 self.assertFalse(self.simple.typed(key))
 
-    def test_jsonfile(self):
-        cfg = LayeredConfig(self.simple)
-        self._test_mainsection(cfg,
-                               int_type=int,
-                               bool_type=bool,
-                               list_type=list,
-                               date_type=str,
-                               datetime_type=str)
-
-    def test_jsonfile_subsections(self):
-        cfg = LayeredConfig(self.complex)
-        self._test_subsections(cfg,
-                               int_type=int,
-                               bool_type=bool,
-                               list_type=list,
-                               date_type=str,
-                               datetime_type=str,
-                               arbitrary_nesting=True)
-
     def test_write(self):
         self.maxDiff = None
         cfg = LayeredConfig(self.complex)
@@ -460,8 +461,7 @@ class TestJSONFile(unittest.TestCase, TestConfigSourceHelper,
         self.assertEqual(want, got)
 
 class TestYAMLFile(unittest.TestCase,
-                   TestConfigSourceHelper,
-                   TestLayeredConfigHelper):
+                   TestConfigSourceHelper):
     def setUp(self):
         with open("simple.yaml", "w") as fp:
             fp.write("""
@@ -501,17 +501,6 @@ extramodule:
         os.unlink("simple.yaml")
         os.unlink("complex.yaml")
 
-    # PyYAML seems to transparently convert strings that look like
-    # dates, booleans, ints etc into the correct python
-    # objects. Wow. 
-    def test_yamlfile(self):
-        cfg = LayeredConfig(self.simple)
-        self._test_mainsection(cfg)
-
-    def test_yamlfile_subsections(self):
-        cfg = LayeredConfig(self.complex)
-        self._test_subsections(cfg)
-
     # Also, strings are unicode when they need to be,
     # str otherwise.
     def test_i18n(self):
@@ -553,8 +542,9 @@ processes: 4
             got = fp.read().replace("\r\n", "\n")
         self.assertEqual(want, got)
 
-class TestPListFile(unittest.TestCase, TestConfigSourceHelper,
-                    TestLayeredConfigHelper):
+class TestPListFile(unittest.TestCase, TestConfigSourceHelper):
+
+    supported_types = (str, int, bool, list, datetime)
 
     def setUp(self):
         with open("simple.plist", "w") as fp:
@@ -641,16 +631,6 @@ class TestPListFile(unittest.TestCase, TestConfigSourceHelper,
         self.assertEqual(self.simple.get("lastrun"),
                          datetime(2014, 10, 15, 14, 32, 7))
 
-    def test_plistfile(self):
-        cfg = LayeredConfig(self.simple)
-        self._test_mainsection(cfg,
-                               date_type=str)
-
-    def test_plistfile_subsections(self):
-        cfg = LayeredConfig(self.complex)
-        self._test_subsections(cfg,
-                               date_type=str)
-
     def test_write(self):
         self.maxDiff = None
         cfg = LayeredConfig(self.complex)
@@ -710,28 +690,80 @@ class TestPListFile(unittest.TestCase, TestConfigSourceHelper,
         self.assertEqual(want, got)
 
 
-class TestCommandline(unittest.TestCase, TestConfigSourceHelper,
-                      TestLayeredConfigHelper):
+class TestPyFile(unittest.TestCase, TestConfigSourceHelper):
 
-    simple = Commandline(['--home=mydata',
-                          '--processes=4',
-                          '--force',  # note implicit boolean typing
-                          '--extra=foo',
-                          '--extra=bar',
-                          '--expires=2014-10-15',
-                          '--lastrun=2014-10-15 14:32:07'])
+    def setUp(self):
+        with open("simple.py", "w") as fp:
+            fp.write("""from __future__ import unicode_literals
+import datetime
+
+expires = datetime.date(2014,10,15)
+extra = ['foo', 'bar']
+force = True
+home = 'mydata'
+lastrun = datetime.datetime(2014,10,15,14,32,7)
+processes = 4
+""")
+        with open("complex.py", "w") as fp:
+            fp.write("""from __future__ import unicode_literals
+import datetime
+
+extra = ['foo', 'bar']
+force = True
+home = 'mydata'
+processes = 4
+
+mymodule = Subsection()
+mymodule.expires = datetime.date(2014,10,15)
+mymodule.extra = ['foo', 'baz']
+mymodule.force = False
+mymodule.arbitrary = Subsection()
+mymodule.arbitrary.nesting = Subsection()
+mymodule.arbitrary.nesting.depth = 'works'
+
+extramodule = Subsection()
+extramodule.unique = True
+""")
+        self.simple = PyFile("simple.py")
+        self.complex = PyFile("complex.py")
+
+    def tearDown(self):
+        os.unlink("simple.py")
+        os.unlink("complex.py")
+
+
+class TestCommandline(unittest.TestCase, TestConfigSourceHelper):
+
+    # Note: bool is "half-way" supported. Only value-less parameters
+    # are typed as bool (eg "--force", not "--force=True")
+    supported_types = (str, list, bool)
+
+    simple_cmdline = ['--home=mydata',
+                      '--processes=4',
+                      '--force',  # note implicit boolean typing
+                      '--extra=foo',
+                      '--extra=bar',
+                      '--expires=2014-10-15',
+                      '--lastrun=2014-10-15 14:32:07']
+
+    complex_cmdline = ['--home=mydata',
+                       '--processes=4',
+                       '--force=True',
+                       '--extra=foo',
+                       '--extra=bar',
+                       '--mymodule-force=False',
+                       '--mymodule-extra=foo',
+                       '--mymodule-extra=baz',
+                       '--mymodule-expires=2014-10-15',
+                       '--mymodule-arbitrary-nesting-depth=works',
+                       '--extramodule-unique']
+
+    def setUp(self):
+        super(TestCommandline, self).setUp()
+        # this means we lack typing information
+        self.simple = Commandline(self.simple_cmdline)
+        self.complex = Commandline(self.complex_cmdline)
 
-    complex = Commandline(['--home=mydata',
-                           '--processes=4',
-                           '--force=True',
-                           '--extra=foo',
-                           '--extra=bar',
-                           '--mymodule-force=False',
-                           '--mymodule-extra=foo',
-                           '--mymodule-extra=baz',
-                           '--mymodule-expires=2014-10-15',
-                           '--mymodule-arbitrary-nesting-depth=works',
-                           '--extramodule-unique'])
 
     # Overrides of TestHelper.test_get, .test_typed and and due to
     # limitations of Commandline (carries almost no typeinfo)
@@ -739,7 +771,7 @@ class TestCommandline(unittest.TestCase, TestConfigSourceHelper,
         self.assertEqual(self.simple.get("home"), "mydata")
         self.assertEqual(self.simple.get("processes"), "4")
         self.assertEqual(self.simple.get("force"), True)
-        self.assertEqual(self.simple.get("extra"), ['foo','bar']) # note typed!
+        self.assertEqual(self.simple.get("extra"), ['foo','bar'])  # note typed!
         self.assertEqual(self.simple.get("expires"), "2014-10-15")
         self.assertEqual(self.simple.get("lastrun"), "2014-10-15 14:32:07")
 
@@ -749,29 +781,64 @@ class TestCommandline(unittest.TestCase, TestConfigSourceHelper,
             if key in ("force", "extra"):
                 self.assertTrue(self.simple.typed(key))
             else:
-                self.assertFalse(self.simple.typed(key))                
-
-    def test_commandline(self):
-        cfg = LayeredConfig(self.simple)
-        self._test_mainsection(cfg,
-                               int_type=str,
-                               date_type=str,
-                               datetime_type=str, 
-                               bool_type=bool,  # not always, see below
-                               list_type=list)
-
-    def test_commandline_subsections(self):
-        cfg = LayeredConfig(self.complex)
-        self._test_subsections(cfg,
-                               int_type=str,
-                               bool_type=str,  # "--mymodule-force=False"
-                               list_type=list,
-                               date_type=str,
-                               datetime_type=str)
+                self.assertFalse(self.simple.typed(key))
 
 
-class TestEnvironment(unittest.TestCase, TestConfigSourceHelper, TestLayeredConfigHelper):
+    def test_config_subsections(self):
+        # this case uses valued parameter for --force et al, which
+        # cannot be reliably converted to bools using only intrinsic
+        # information
+        self.supported_types = (str, list)
+        super(TestCommandline, self).test_config_subsections()
 
+
+class TestCommandlineConfigured(TestCommandline):
+
+    supported_types = (str, int, bool, date, datetime, list)
+    
+    def setUp(self):
+        super(TestCommandlineConfigured, self).setUp()
+        simp = argparse.ArgumentParser(description="This is a simple program")
+        simp.add_argument('--home', help="The home directory of the app")
+        simp.add_argument('--processes', type=int, help="Number of simultaneous processes")
+        simp.add_argument('--force', type=LayeredConfig.boolconvert, nargs='?', const=True)
+        simp.add_argument('--extra', action='append')
+        simp.add_argument('--expires', type=LayeredConfig.dateconvert)
+        simp.add_argument('--lastrun', type=LayeredConfig.datetimeconvert)
+        simp.add_argument('--unused')
+        self.simple = Commandline(self.simple_cmdline,
+                                  parser=simp)
+
+        comp = argparse.ArgumentParser(description="This is a complex program")
+        comp.add_argument('--home', help="The home directory of the app")
+        comp.add_argument('--processes', type=int, help="Number of simultaneous processes")
+        comp.add_argument('--force', type=LayeredConfig.boolconvert, nargs='?', const=True)
+        comp.add_argument('--extra', action='append')
+        comp.add_argument('--mymodule-force', type=LayeredConfig.boolconvert, nargs='?', const=True)
+        comp.add_argument('--mymodule-extra', action='append')
+        comp.add_argument('--mymodule-expires', type=LayeredConfig.dateconvert)
+        comp.add_argument('--mymodule-arbitrary-nesting-depth')
+        comp.add_argument('--extramodule-unique', nargs='?', const=True)
+        self.complex = Commandline(self.complex_cmdline,
+                                   parser=comp)
+
+    def test_get(self):
+        # re-enable the original impl of test_get
+        TestConfigSourceHelper.test_get(self)
+
+    def test_config_subsections(self):
+        # re-enable the original impl of test_config_subsections
+        TestConfigSourceHelper.test_config_subsections(self)
+
+    def test_typed(self):
+        # re-enable the original impl of test_get
+        TestConfigSourceHelper.test_typed(self)
+
+
+class TestEnvironment(unittest.TestCase, TestConfigSourceHelper):
+
+    supported_types = (str,)
+    
     simple = Environment({'MYAPP_HOME': 'mydata',
                           'MYAPP_PROCESSES': '4',
                           'MYAPP_FORCE': 'True',
@@ -802,25 +869,189 @@ class TestEnvironment(unittest.TestCase, TestConfigSourceHelper, TestLayeredConf
         for key in self.simple.keys():
             self.assertFalse(self.simple.typed(key))
 
-    def test_environment(self):
-        cfg = LayeredConfig(self.simple)
-        self._test_mainsection(cfg,
-                               int_type=str,
-                               bool_type=str,
-                               list_type=str,
-                               date_type=str,
-                               datetime_type=str)
 
-    def test_environment_subsections(self):
+# NB: This assumes that an etcd daemon is running with default
+# settings
+ETCD_BASE = "http://127.0.0.1:4001/v2/keys"
+
+@unittest.skipIf("APPVEYOR" in os.environ,
+                 "Not running etcd dependent tests on Appveyor")
+class TestEtcdStore(unittest.TestCase, TestConfigSourceHelper):
+    maxDiff = None
+    def strlower(value):
+        return str(value).lower()
+
+    supported_types = (str,)
+    transforms = {bool: strlower}
+
+    def _clear_server(self):
+        resp = requests.get(ETCD_BASE + "/")
+        resp.raise_for_status()
+        json = resp.json()
+        if 'nodes' in json['node']:
+            for node in json['node']['nodes']:
+                resp = requests.delete(ETCD_BASE + "%s?recursive=true" % node['key'])
+
+    @property
+    def simple(self):
+        self._clear_server()
+        requests.put(ETCD_BASE + "/home", data={'value': 'mydata'})
+        requests.put(ETCD_BASE + "/processes", data={'value': '4'})
+        requests.put(ETCD_BASE + "/force", data={'value': "True"})
+        requests.put(ETCD_BASE + "/extra", data={'value': "foo, bar"})
+        requests.put(ETCD_BASE + "/expires", data={'value': "2014-10-15"})
+        requests.put(ETCD_BASE + "/lastrun", data={'value': "2014-10-15 14:32:07"})
+        return EtcdStore() 
+
+    @property
+    def complex(self):
+        self._clear_server()
+        requests.put(ETCD_BASE + "/home", data={'value': "mydata"})
+        requests.put(ETCD_BASE + "/processes", data={'value': "4"})
+        requests.put(ETCD_BASE + "/force", data={'value': "True"})
+        requests.put(ETCD_BASE + "/extra", data={'value': "foo, bar"})
+        requests.put(ETCD_BASE + "/mymodule/force", data={'value': "False"})
+        requests.put(ETCD_BASE + "/mymodule/extra", data={'value': "foo, baz"})
+        requests.put(ETCD_BASE + "/mymodule/expires", data={'value': "2014-10-15"})
+        requests.put(ETCD_BASE + "/mymodule/arbitrary/nesting/depth", data={'value': "works"})
+        requests.put(ETCD_BASE + "/extramodule/unique", data={'value': "True"})
+        return EtcdStore()
+
+    def test_typed(self):
+        for key in self.simple.keys():
+            self.assertFalse(self.simple.typed(key))
+
+    def test_get(self):
+        # FIXME: This test should be able to look at supported_types
+        # like test_singlesection and test_subsections do, so derived
+        # testcase classes don't need to override it.
+        conf = self.simple
+        self.assertEqual(conf.get("home"), "mydata")
+        self.assertEqual(conf.get("processes"), "4")
+        self.assertEqual(conf.get("force"), "true")
+        self.assertEqual(conf.get("extra"), "foo, bar")
+        self.assertEqual(conf.get("expires"), "2014-10-15")
+        self.assertEqual(conf.get("lastrun"), "2014-10-15 14:32:07")
+
+    def test_write(self):
+        def indexfilter(node):
+            # remove keys like createdIdex / modifiedIndex whose
+            # values always changes
+            if isinstance(node, dict):
+                for key in list(node.keys()):
+                    if key == "nodes":
+                        indexfilter(node[key])
+                    elif key.endswith("Index"):
+                        del node[key]
+            else:
+                node[:] = sorted(node, key=itemgetter('key'))
+                for subnode in node:
+                    indexfilter(subnode)
+
         cfg = LayeredConfig(self.complex)
-        self._test_subsections(cfg,
-                               int_type=str,
-                               bool_type=str,
-                               list_type=str,
-                               date_type=str,
-                               datetime_type=str)
+        cfg.mymodule.expires = date(2014, 10, 24)
+        LayeredConfig.write(cfg.mymodule)
+        want = """
+{
+    "dir": true,
+    "key": "/",
+    "nodes": [
+        {
+            "createdIndex": 4627,
+            "key": "/home",
+            "modifiedIndex": 4627,
+            "value": "mydata"
+        },
+        {
+            "createdIndex": 4628,
+            "key": "/processes",
+            "modifiedIndex": 4628,
+            "value": "4"
+        },
+        {
+            "createdIndex": 4629,
+            "key": "/force",
+            "modifiedIndex": 4629,
+            "value": "true"
+        },
+        {
+            "createdIndex": 4630,
+            "key": "/extra",
+            "modifiedIndex": 4630,
+            "value": "foo, bar"
+        },
+        {
+            "createdIndex": 4631,
+            "dir": true,
+            "key": "/mymodule",
+            "modifiedIndex": 4631,
+            "nodes": [
+                {
+                    "createdIndex": 4631,
+                    "key": "/mymodule/force",
+                    "modifiedIndex": 4631,
+                    "value": "false"
+                },
+                {
+                    "createdIndex": 4632,
+                    "key": "/mymodule/extra",
+                    "modifiedIndex": 4632,
+                    "value": "foo, baz"
+                },
+                {
+                    "createdIndex": 4633,
+                    "key": "/mymodule/expires",
+                    "modifiedIndex": 4633,
+                    "value": "2014-10-24"
+                },
+                {
+                    "createdIndex": 4634,
+                    "dir": true,
+                    "key": "/mymodule/arbitrary",
+                    "modifiedIndex": 4634,
+                    "nodes": [
+                        {
+                            "createdIndex": 4634,
+                            "dir": true,
+                            "key": "/mymodule/arbitrary/nesting",
+                            "modifiedIndex": 4634,
+                            "nodes": [
+                                {
+                                    "createdIndex": 4634,
+                                    "key": "/mymodule/arbitrary/nesting/depth",
+                                    "modifiedIndex": 4634,
+                                    "value": "works"
+                                }
+                            ]
+                        }
+                    ]
+                }
+            ]
+        },
+        {
+            "createdIndex": 4635,
+            "dir": true,
+            "key": "/extramodule",
+            "modifiedIndex": 4635,
+            "nodes": [
+                {
+                    "createdIndex": 4635,
+                    "key": "/extramodule/unique",
+                    "modifiedIndex": 4635,
+                    "value": "true"
+                }
+            ]
+        }
+    ]
+}"""
+        want = json.loads(want)
+        indexfilter(want)
+        got = requests.get("http://localhost:4001/v2/keys/?recursive=true").json()['node']
+        indexfilter(got)
+        self.assertEqual(want, got)
 
 
+        
 class TestTyping(unittest.TestCase, TestLayeredConfigHelper):
     types = {'home': str,
              'processes': int,
@@ -832,7 +1063,7 @@ class TestTyping(unittest.TestCase, TestLayeredConfigHelper):
                           'lastrun': datetime,
                       }
              }
-    
+
     def test_typed_commandline(self):
         cmdline = ['--home=mydata',
                    '--processes=4',
@@ -847,7 +1078,7 @@ class TestTyping(unittest.TestCase, TestLayeredConfigHelper):
                    '--mymodule-arbitrary-nesting-depth=works',
                    '--extramodule-unique']
         cfg = LayeredConfig(Defaults(self.types), Commandline(cmdline))
-        self._test_subsections(cfg)
+        self._test_config_subsections(cfg)
         self.assertTrue(cfg.implicitboolean)
         self.assertIs(type(cfg.implicitboolean), bool)
 
@@ -898,9 +1129,8 @@ class TestTyping(unittest.TestCase, TestLayeredConfigHelper):
 
 
 class TestTypingINIFile(TestINIFileHelper,
-                        unittest.TestCase,
-                        TestLayeredConfigHelper):
-    
+                        TestLayeredConfigHelper,
+                        unittest.TestCase):
     types = {'home': str,
              'processes': int,
              'force': bool,
@@ -911,10 +1141,15 @@ class TestTypingINIFile(TestINIFileHelper,
                           'lastrun': datetime,
                       }
              }
-    
+
+    # FIXME: find a neat way to run the tests in
+    # test_config_subsections with a LayeredConfig object that uses a
+    # Default object for typing
     def test_typed_inifile(self):
         cfg = LayeredConfig(Defaults(self.types), INIFile("complex.ini"))
-        self._test_subsections(cfg, arbitrary_nesting=False)
+        self.supported_types = (str, bool, int, list, date, datetime)
+        self.supports_nesting = False
+        self._test_config_subsections(cfg)
 
 
 class TestLayered(TestINIFileHelper, unittest.TestCase):
@@ -931,7 +1166,7 @@ class TestLayered(TestINIFileHelper, unittest.TestCase):
         self.assertEqual(cfg.home, 'yourdata')
         cfg = LayeredConfig(Defaults(defaults), INIFile("simple.ini"),
                             Environment(env, prefix="MYAPP_"),
-                            Commandline(cmdline))
+                           Commandline(cmdline))
         self.assertEqual(cfg.home, 'anotherplace')
         self.assertEqual(['home', 'processes', 'force', 'extra', 'expires',
                           'lastrun'], list(cfg))
@@ -1008,8 +1243,6 @@ class TestModifications(TestINIFileHelper, unittest.TestCase):
         # this shouldn't, either
         self.globalconf.base.download_text = "WHAT"
 
-        
-
     def test_write_noconfigfile(self):
         cfg = LayeredConfig(Defaults({'lastrun':
                                       datetime(2012, 9, 18, 15, 41, 0)}))
